"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var image_service_1 = require("../image.service");
var ng2_dragula_1 = require("ng2-dragula");
var Observable_1 = require("rxjs/Observable");
require("rxjs/add/observable/from");
var FileHolder = (function () {
    function FileHolder(src, file, id) {
        this.src = src;
        this.file = file;
        this.id = id;
        this.pending = false;
    }
    return FileHolder;
}());
exports.FileHolder = FileHolder;
var ImageUploadComponent = (function () {
    function ImageUploadComponent(imageService, dragulaService) {
        var _this = this;
        this.imageService = imageService;
        this.dragulaService = dragulaService;
        this.icandelete = false;
        this.max = 100;
        this.preview = true;
        this.withCredentials = false;
        this.isPending = new core_1.EventEmitter();
        this.onFileUploadFinish = new core_1.EventEmitter();
        this.onFilesUploadFinish = new core_1.EventEmitter();
        this.onOrderChanged = new core_1.EventEmitter();
        this.onRemove = new core_1.EventEmitter();
        this.files = [];
        this.showFileTooLargeMessage = false;
        this.fileCounter = 0;
        this.pendingFilesCounter = 0;
        this.isFileOver = false;
        this.buttonCaption = 'Select Images';
        this.dropBoxMessage = 'Drop your images here!';
        this.supportedExtensions = ['image/*'];
        dragulaService.setOptions('first-bag', {
            direction: 'horizontal'
        });
        dragulaService.dropModel.subscribe(function (value) {
            var arr = [];
            for (var i in _this.files) {
                arr.push(_this.files[i].id);
            }
            console.log(arr);
            _this.onOrderChanged.emit(arr);
        });
    }
    ImageUploadComponent.prototype.ngOnDestroy = function () {
        this.dragulaService.destroy('first-bag');
    };
    ImageUploadComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.imageService.setUrl(this.url);
        if (!this.fileTooLargeMessage) {
            this.fileTooLargeMessage = 'An image was too large and was not uploaded.' + (this.maxFileSize
                ? (' The maximum file size is ' + this.maxFileSize / 1024) + 'KiB.'
                : '');
        }
        if (this.supportedExtensions) {
            this.supportedExtensions = this.supportedExtensions.map(function (ext) { return 'image/' + ext; });
        }
        if (typeof this.uploaded !== 'undefined')
            if (this.uploaded.length > 0)
                for (var i = 0; i < this.uploaded.length; i++) {
                    if (!this.uploaded[i]['imageUrl']) {
                        continue;
                    }
                    this.imageService.convertFileToDataURLviaFileReader(this.uploaded[i]['imageUrl'], this.uploaded[i]['id'], this.headers).subscribe(function (response) {
                        var fileHolder = new FileHolder(response.image, new File([response.image], response.id), response.id);
                        _this.files.push(fileHolder);
                        _this.max--;
                    }, function (error) { return console.warn(error); });
                }
    };
    ImageUploadComponent.prototype.fileChange = function (files) {
        var remainingSlots = this.countRemainingSlots();
        var filesToUploadNum = files.length > remainingSlots ? remainingSlots : files.length;
        if (this.url && filesToUploadNum != 0) {
            this.isPending.emit(true);
        }
        this.fileCounter += filesToUploadNum;
        this.showFileTooLargeMessage = false;
        this.uploadFiles(files, filesToUploadNum);
    };
    ImageUploadComponent.prototype.deleteAll = function () {
        this.files = [];
        this.fileCounter = 0;
    };
    ImageUploadComponent.prototype.uploadFiles = function (files, filesToUploadNum) {
        var _this = this;
        var uploadedFilesCount = 0;
        var _loop_1 = function (i) {
            var file = files[i];
            if (this_1.maxFileSize && file.size > this_1.maxFileSize) {
                this_1.showFileTooLargeMessage = true;
                return "continue";
            }
            var img = document.createElement('img');
            img.src = window.URL.createObjectURL(file);
            var reader = new FileReader();
            reader.addEventListener('load', function (event) {
                var fileHolder = new FileHolder(event.target.result, file);
                fileHolder.serverResponse = "good boy: " + i;
                _this.uploadSingleFile(fileHolder).subscribe(function (data) {
                    if (data.result === 'ok') {
                        fileHolder.id = data.response.id;
                        uploadedFilesCount++;
                        if (uploadedFilesCount === filesToUploadNum) {
                            _this.onFilesUploadFinish.emit(_this.files);
                        }
                    }
                });
                _this.files.push(fileHolder);
            }, false);
            reader.readAsDataURL(file);
        };
        var this_1 = this;
        for (var i = 0; i < filesToUploadNum; i++) {
            _loop_1(i);
        }
    };
    ImageUploadComponent.prototype.uploadSingleFile = function (fileHolder) {
        var _this = this;
        return Observable_1.Observable.create(function (observer) {
            if (_this.url) {
                _this.pendingFilesCounter++;
                fileHolder.pending = true;
                _this.imageService.postImage(fileHolder.file, _this.headers).subscribe(function (response) {
                    fileHolder.serverResponse = response;
                    _this.onFileUploadFinish.emit(fileHolder);
                    fileHolder.pending = false;
                    if (--_this.pendingFilesCounter == 0) {
                        _this.isPending.emit(false);
                    }
                    observer.next({ 'result': 'ok', 'response': JSON.parse(response) });
                    observer.complete();
                });
            }
            else {
                _this.onFileUploadFinish.emit(fileHolder);
                observer.next({ 'result': 'error' });
                observer.complete();
            }
        });
    };
    ImageUploadComponent.prototype.deleteFile = function (file) {
        var _this = this;
        this.imageService.deleteImage(this.url, file.id, this.headers)
            .subscribe(function (res) {
            _this.onRemove.emit(file);
            var index = _this.files.indexOf(file);
            _this.files.splice(index, 1);
            _this.fileCounter--;
        });
    };
    ImageUploadComponent.prototype.fileOver = function (isOver) {
        this.isFileOver = isOver;
    };
    ImageUploadComponent.prototype.countRemainingSlots = function () {
        return this.max - this.fileCounter;
    };
    Object.defineProperty(ImageUploadComponent.prototype, "value", {
        get: function () {
            return this.files;
        },
        enumerable: true,
        configurable: true
    });
    ImageUploadComponent.decorators = [
        { type: core_1.Component, args: [{
                    selector: 'image-upload',
                    template: "\n    <div class=\"image-upload\"\n         fileDrop\n         [accept]=\"['image/*']\"\n         (isFileOver)=\"fileOver($event)\"\n         (fileDrop)=\"fileChange($event)\"\n         [ngClass]=\"{'file-is-over': isFileOver}\"\n    >\n      <div class=\"file-upload hr-inline-group\">\n        <div class=\"drag-box-message\">{{dropBoxMessage}}</div>\n        <label class=\"upload-button\">\n          <span>{{buttonCaption}}</span>\n          <input\n            type=\"file\"\n            accept=\"image/*\"\n            multiple (change)=\"fileChange(input.files)\"\n            #input>\n        </label>\n      </div>\n\n      <div *ngIf=\"preview\" class=\"image-container hr-inline-group\" [dragula]='\"first-bag\"' [dragulaModel]='files'>\n        <div\n          class=\"image\"\n          *ngFor=\"let file of files\" [id]=\"file.id\"\n          [ngStyle]=\"{'background-image': 'url('+ file.src +')'}\"\n        >\n          <div *ngIf=\"file.pending\" class=\"loading-overlay\">\n            <div class=\"spinningCircle\"></div>\n          </div>\n          <div *ngIf=\"!file.pending\" class=\"x-mark\" (click)=\"deleteFile(file)\">\n            <span class=\"close\"></span>\n          </div>\n        </div>\n      </div>\n    </div>\n  ",
                    styles: ["\n    .image-upload {\n      --common-radius: 3px;\n      --active-color: #33CC99;\n      position: relative;\n      border-radius: var(--common-radius);\n      border: #d0d0d0 dashed 1px;\n      font-family: sans-serif;\n    }\n\n    .file-is-over {\n      border-color: var(--active-color);\n      border-style: solid;\n    }\n\n    .hr-inline-group:after {\n      display: table;\n      clear: both;\n      content: \"\";\n    }\n\n    .file-upload {\n      padding: 16px;\n      background-color: #f8f8f8;\n    }\n\n    .drag-box-message {\n      float: left;\n      display: inline-block;\n      margin-left: 12px;\n      padding-top: 14px;\n      color: #9b9b9b;\n      font-weight: 600;\n    }\n\n    label.upload-button input[type=file] {\n      display: none;\n      position: fixed;\n      top: -99999px;\n    }\n\n    .upload-button {\n      cursor: pointer;\n      background-color: var(--active-color);\n      padding: 10px;\n      color: white;\n      font-size: 1.25em;\n      font-weight: 500;\n      text-transform: uppercase;\n      display: inline-block;\n      float: left;\n\n      -webkit-box-shadow: 2px 2px 4px 0px rgba(148, 148, 148, 0.6);\n      -moz-box-shadow: 2px 2px 4px 0px rgba(148, 148, 148, 0.6);\n      box-shadow: 2px 2px 4px 0px rgba(148, 148, 148, 0.6);\n    }\n\n    .upload-button:active span {\n      position: relative;\n      display: block;\n      top: 1px;\n    }\n\n    .image-container {\n      background-color: #fdfdfd;\n      padding: 0 10px 0 10px;\n    }\n\n    .image {\n      cursor: move; /* fallback if grab cursor is unsupported */\n      cursor: grab;\n      border: 1px solid #fdfdfd;\n      display: inline-block;\n      margin: 6px;\n      width: 86px;\n      height: 86px;\n      background: center center no-repeat;\n      background-size: contain;\n      position: relative;\n    }\n\n    .x-mark {\n      width: 20px;\n      height: 20px;\n      text-align: center;\n      cursor: pointer;\n      border-radius: 50%;\n      background-color: black;\n      opacity: .7;\n      color: white;\n      position: absolute;\n      left: 0;\n      right: 0;\n      margin-left: auto;\n      margin-right: auto;\n      top: 50%;\n      -webkit-transform: translateY(-50%);\n      -moz-transform: translateY(-50%);\n      -ms-transform: translateY(-50%);\n      -o-transform: translateY(-50%);\n      transform: translateY(-50%);\n    }\n\n    .close {\n      width: 20px;\n      height: 20px;\n      opacity: .7;\n      position: relative;\n      padding-right: 3px;\n    }\n\n    .x-mark:hover .close {\n      opacity: 1;\n    }\n\n    .close:before, .close:after {\n      border-radius: 2px;\n      position: absolute;\n      content: '';\n      height: 16px;\n      width: 2px;\n      top: 2px;\n      background-color: #FFFFFF;\n    }\n\n    .close:before {\n      transform: rotate(45deg);\n    }\n\n    .close:after {\n      transform: rotate(-45deg);\n    }\n\n    .loading-overlay {\n      position: absolute;\n      top: 0;\n      left: 0;\n      bottom: 0;\n      right: 0;\n      background-color: black;\n      opacity: .7;\n    }\n\n    .spinningCircle {\n      height: 30px;\n      width: 30px;\n      margin: auto;\n      position: absolute;\n      top: 0;\n      left: 0;\n      bottom: 0;\n      right: 0;\n      border-radius: 50%;\n      border: 3px solid rgba(255, 255, 255, 0);\n      border-top: 3px solid white;\n      border-right: 3px solid white;\n      -webkit-animation: spinner 2s infinite cubic-bezier(0.085, 0.625, 0.855, 0.360);\n      animation: spinner 2s infinite cubic-bezier(0.085, 0.625, 0.855, 0.360);\n    }\n\n    @-webkit-keyframes spinner {\n      0% {\n        -webkit-transform: rotate(0deg);\n        transform: rotate(0deg);\n      }\n      100% {\n        -webkit-transform: rotate(360deg);\n        transform: rotate(360deg);\n      }\n    }\n\n    @keyframes spinner {\n      0% {\n        -webkit-transform: rotate(0deg);\n        transform: rotate(0deg);\n      }\n      100% {\n        -webkit-transform: rotate(360deg);\n        transform: rotate(360deg);\n\n      }\n    }\n\n    .gu-mirror {\n      cursor: grabbing !important;\n      cursor: -moz-grabbing !important;\n      cursor: -webkit-grabbing !important;\n      position: fixed !important;\n      margin: 0 !important;\n      z-index: 9999 !important;\n      opacity: .8;\n      -ms-filter: \"progid:DXImageTransform.Microsoft.Alpha(Opacity=80)\";\n      filter: alpha(opacity=80)\n    }\n\n    .gu-hide {\n      display: none !important\n    }\n\n    .gu-unselectable {\n      -webkit-user-select: none !important;\n      -moz-user-select: none !important;\n      -ms-user-select: none !important;\n      user-select: none !important\n    }\n\n    .gu-transit {\n      opacity: .2;\n      -ms-filter: \"progid:DXImageTransform.Microsoft.Alpha(Opacity=20)\";\n      filter: alpha(opacity=20)\n    }\n  "]
                },] },
    ];
    ImageUploadComponent.ctorParameters = function () { return [
        { type: image_service_1.ImageService, },
        { type: ng2_dragula_1.DragulaService, },
    ]; };
    ImageUploadComponent.propDecorators = {
        'uploaded': [{ type: core_1.Input },],
        'icandelete': [{ type: core_1.Input },],
        'max': [{ type: core_1.Input },],
        'url': [{ type: core_1.Input },],
        'headers': [{ type: core_1.Input },],
        'preview': [{ type: core_1.Input },],
        'maxFileSize': [{ type: core_1.Input },],
        'withCredentials': [{ type: core_1.Input },],
        'partName': [{ type: core_1.Input },],
        'isPending': [{ type: core_1.Output },],
        'onFileUploadFinish': [{ type: core_1.Output },],
        'onFilesUploadFinish': [{ type: core_1.Output },],
        'onOrderChanged': [{ type: core_1.Output },],
        'onRemove': [{ type: core_1.Output },],
        'buttonCaption': [{ type: core_1.Input },],
        'dropBoxMessage': [{ type: core_1.Input },],
        'fileTooLargeMessage': [{ type: core_1.Input },],
        'supportedExtensions': [{ type: core_1.Input, args: ['extensions',] },],
    };
    return ImageUploadComponent;
}());
exports.ImageUploadComponent = ImageUploadComponent;
