"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var Observable_1 = require("rxjs/Observable");
var http_1 = require("@angular/http");
var ImageService = (function () {
    function ImageService(http) {
        this.http = http;
        this.deleteEmit = new core_1.EventEmitter();
    }
    ImageService.prototype.setUrl = function (url) {
        this.url = url;
    };
    ImageService.prototype.postImage = function (image, headers) {
        var _this = this;
        this.checkUrl();
        return Observable_1.Observable.create(function (observer) {
            var formData = new FormData();
            var xhr = new XMLHttpRequest();
            formData.append('image', image);
            xhr.onreadystatechange = function () {
                if (xhr.readyState === 4) {
                    if (xhr.status === 200) {
                        observer.next(xhr.response);
                        observer.complete();
                    }
                    else {
                        observer.error(xhr.response);
                    }
                }
            };
            xhr.open('POST', _this.url, true);
            if (headers)
                for (var _i = 0, headers_1 = headers; _i < headers_1.length; _i++) {
                    var header = headers_1[_i];
                    xhr.setRequestHeader(header.header, header.value);
                }
            xhr.send(formData);
        });
    };
    ImageService.prototype.deleteImage = function (url, id, headers) {
        this.checkUrl();
        var hs = new http_1.Headers();
        for (var _i = 0, headers_2 = headers; _i < headers_2.length; _i++) {
            var header = headers_2[_i];
            hs.append(header.header, header.value);
        }
        return this.http.delete(url, { params: { id: id }, headers: hs });
    };
    ImageService.prototype.convertFileToDataURLviaFileReader = function (url, id, headers) {
        return Observable_1.Observable.create(function (observer) {
            var xhr = new XMLHttpRequest();
            xhr.responseType = 'blob';
            xhr.onreadystatechange = function () {
                var reader = new FileReader();
                reader.onloadend = function () {
                    observer.next({ 'image': reader.result, id: id });
                    observer.complete();
                };
                if (xhr.response != null) {
                    reader.readAsDataURL(xhr.response);
                }
            };
            xhr.open('GET', url);
            if (headers)
                for (var _i = 0, headers_3 = headers; _i < headers_3.length; _i++) {
                    var header = headers_3[_i];
                    xhr.setRequestHeader(header.header, header.value);
                }
            xhr.send();
        });
    };
    ImageService.prototype.checkUrl = function () {
        if (!this.url) {
            throw new Error('Url is not set! Please use setUrl(url) method before doing queries');
        }
    };
    ImageService.decorators = [
        { type: core_1.Injectable },
    ];
    ImageService.ctorParameters = function () { return [
        { type: http_1.Http, },
    ]; };
    return ImageService;
}());
exports.ImageService = ImageService;
